@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 1.8.7   2020/05/05
+ * Version: 1.8.9   2021/04/01
  */
 
 #include "internal.h"
@@ -379,7 +379,15 @@ static char *ccs_get_local_path(struct dentry *dentry, char * const buffer,
 	if (sb->s_magic == PROC_SUPER_MAGIC && *pos == '/') {
 		char *ep;
 		const pid_t pid = (pid_t) simple_strtoul(pos + 1, &ep, 10);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 8, 0)
+		if (*ep == '/' && pid && pid ==
+		    task_tgid_nr_ns(current, proc_pid_ns(sb))) {
+			pos = ep - 5;
+			if (pos < buffer)
+				goto out;
+			memmove(pos, "/self", 5);
+		}
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
 		if (*ep == '/' && pid && pid ==
 		    task_tgid_nr_ns(current, sb->s_fs_info)) {
 			pos = ep - 5;
@@ -502,26 +510,24 @@ char *ccs_realpath(const struct path *path)
 			pos = ccs_get_absolute_path(path, buf, buf_len - 1);
 			ccs_realpath_unlock();
 		}
-		if (pos == ERR_PTR(-EINVAL))
-			pos = ccs_get_local_path(path->dentry, buf,
-						 buf_len - 1);
 #elif LINUX_VERSION_CODE < KERNEL_VERSION(4, 9, 0)
 		if (!path->mnt ||
 		    (!inode->i_op->rename && !inode->i_op->rename2 &&
 		     !(sb->s_type->fs_flags & FS_REQUIRES_DEV)))
-			pos = ccs_get_local_path(path->dentry, buf,
-						 buf_len - 1);
+			pos = ERR_PTR(-EINVAL);
 		else
 			pos = ccs_get_absolute_path(path, buf, buf_len - 1);
 #else
 		if (!path->mnt ||
 		    (!inode->i_op->rename &&
 		     !(sb->s_type->fs_flags & FS_REQUIRES_DEV)))
-			pos = ccs_get_local_path(path->dentry, buf,
-						 buf_len - 1);
+			pos = ERR_PTR(-EINVAL);
 		else
 			pos = ccs_get_absolute_path(path, buf, buf_len - 1);
 #endif
+		if (pos == ERR_PTR(-EINVAL))
+			pos = ccs_get_local_path(path->dentry, buf,
+						 buf_len - 1);
 encode:
 		if (IS_ERR(pos))
 			continue;
@@ -689,6 +695,9 @@ const char *ccs_get_exe(void)
 	const char *cp;
 	if (!mm)
 		return NULL;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 1, 0)
+	exe_file = get_mm_exe_file(mm);
+#else
 	down_read(&mm->mmap_sem);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 26)
 	/* Not using get_mm_exe_file() as it is not exported. */
@@ -704,6 +713,7 @@ const char *ccs_get_exe(void)
 	if (exe_file)
 		get_file(exe_file);
 	up_read(&mm->mmap_sem);
+#endif
 	if (!exe_file)
 		return NULL;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 20)
